// $Id: J4MagneticFieldManager.hh,v 1.1.1.1 2002/05/02 07:56:58 hoshina Exp $
#ifndef __J4MAGNETICFIELDMANAGER__
#define __J4MAGNETICFIELDMANAGER__
//*************************************************************************
//* --------------------
//* J4MagneticFieldManager
//* --------------------
//* (Description)
//* 	This class applies magnetic field.
//*     Mother class : G4MagneticField
//*    
//* (Update Record)
//*	2002/09/13  T.Aso	Original version.
//*************************************************************************
 
#include "globals.hh"
#include "G4MagneticField.hh"
#include "J4AttMFieldMap.hh"

// ====================================================================
// -----------------------
// class definition
// -----------------------
 
class J4MagneticFieldManager : public G4MagneticField {

public: 
  inline static J4MagneticFieldManager* GetInstance();

public:
  ~J4MagneticFieldManager() { }
  virtual void GetFieldValue(const  G4double Point[3], 
			     G4double* Bfield ) const;  

  inline void AddMField(J4AttMFieldMap* map);
  inline void RemoveMField(J4AttMFieldMap* map);

  inline void NameList();

protected:
  J4MagneticFieldManager();

private:
  static J4MagneticFieldManager* fpInstance;

  int  fcurCount;
  J4AttMFieldMap** fmagCollection;

};

inline J4MagneticFieldManager* J4MagneticFieldManager::GetInstance () {
  if(!fpInstance) fpInstance=new J4MagneticFieldManager();
  return fpInstance;
}

inline void J4MagneticFieldManager::AddMField(J4AttMFieldMap* map){
  fmagCollection[fcurCount++]  = map; 
}

inline void J4MagneticFieldManager::RemoveMField(J4AttMFieldMap* map){
  for ( G4int i = 0; i < fcurCount ; i++ ){
    if ( fmagCollection[i] == map ) fmagCollection[i] = 0;
  }
}
inline void J4MagneticFieldManager::NameList () {
  G4cout << " ---J4MagneticFieldManager:: NameList()--- "<< fcurCount <<G4endl; 
  for ( G4int i = 0; i < fcurCount ; i++){
    G4String* name = fmagCollection[i]->GetMFieldName();
    G4cout << i <<" "<< *name << G4endl;
  }
};
#endif
